@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -551,17 +552,17 @@ bool
 grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   const grn_id range_id = grn_obj_get_range(ctx, obj);
   if (grn_id_is_builtin(ctx, range_id)) {
-    return GRN_FALSE;
+    return false;
   }
 
   grn_obj *range = range = grn_ctx_at(ctx, range_id);
   if (!range) {
-    return GRN_FALSE;
+    return false;
   }
 
   uint8_t range_type = range->header.type;
@@ -657,12 +658,12 @@ grn_obj_is_value_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_VALUE;
@@ -674,12 +675,12 @@ grn_obj_is_score_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_SCORE;
@@ -716,12 +717,12 @@ grn_obj_is_nsubrecs_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_NSUBRECS;
@@ -733,12 +734,12 @@ grn_obj_is_max_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_MAX;
@@ -750,12 +751,12 @@ grn_obj_is_min_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_MIN;
@@ -767,12 +768,12 @@ grn_obj_is_sum_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_SUM;
@@ -784,12 +785,12 @@ grn_obj_is_avg_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_AVG;
@@ -818,12 +819,12 @@ grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_COLUMN_VALUE;
@@ -861,6 +862,40 @@ grn_obj_is_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
+bool
+grn_obj_is_text_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+    return false;
+  case GRN_ACCESSOR_GET_VALUE:
+    return grn_type_id_is_text_family(ctx,
+                                      grn_obj_get_range(ctx, accessor->obj));
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_text_family_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 bool
 grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -917,82 +952,82 @@ grn_obj_is_vector_accessor(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
-grn_bool
+bool
 grn_obj_is_type(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_TYPE;
 }
 
-grn_bool
+bool
 grn_obj_is_text_family_type(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_type(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return GRN_TYPE_IS_TEXT_FAMILY(grn_obj_id(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_proc(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_PROC;
 }
 
-grn_bool
+bool
 grn_obj_is_tokenizer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_TOKENIZER;
 }
 
-grn_bool
+bool
 grn_obj_is_function_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_FUNCTION;
 }
 
-grn_bool
+bool
 grn_obj_is_selector_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_function_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->callbacks.function.selector != NULL;
 }
 
-grn_bool
+bool
 grn_obj_is_selector_only_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_selector_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
@@ -1010,52 +1045,52 @@ grn_obj_is_applier_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->callbacks.function.applier != NULL;
 }
 
-grn_bool
+bool
 grn_obj_is_normalizer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_NORMALIZER;
 }
 
-grn_bool
+bool
 grn_obj_is_token_filter_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_TOKEN_FILTER;
 }
 
-grn_bool
+bool
 grn_obj_is_scorer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_SCORER;
 }
 
-grn_bool
+bool
 grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
@@ -1075,11 +1110,11 @@ grn_obj_is_aggregator_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->type == GRN_PROC_AGGREGATOR;
 }
 
-grn_bool
+bool
 grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_EXPR;
@@ -1134,13 +1169,14 @@ grn_obj_set_visibility(grn_ctx *ctx, grn_obj *obj, bool is_visible)
   }
 
   if (!(grn_obj_is_index_column(ctx, obj) ||
-        grn_obj_is_token_column(ctx, obj))) {
+        grn_obj_is_token_column(ctx, obj) ||
+        grn_obj_is_generated_column(ctx, obj))) {
     grn_obj inspected;
     GRN_TEXT_INIT(&inspected, 0);
     grn_inspect_limited(ctx, &inspected, obj);
     ERR(GRN_INVALID_ARGUMENT,
         "[obj][set-visibility] "
-        "must be an index column or a token column: <%.*s>",
+        "must be an index column, a token column or a generated column: <%.*s>",
         (int)(GRN_TEXT_LEN(&inspected)),
         GRN_TEXT_VALUE(&inspected));
     GRN_OBJ_FIN(ctx, &inspected);
@@ -1173,7 +1209,33 @@ grn_obj_have_source(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj)
 {
-  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj);
+  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj) &&
+         ((grn_ja *)obj)->generator.length == 0;
+}
+
+bool
+grn_obj_is_generated_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!obj) {
+    return false;
+  }
+
+  grn_raw_string *generator = NULL;
+  switch (obj->header.type) {
+  case GRN_COLUMN_FIX_SIZE:
+    generator = &(((grn_ra *)obj)->generator);
+    break;
+  case GRN_COLUMN_VAR_SIZE:
+    generator = &(((grn_ja *)obj)->generator);
+    break;
+  default:
+    return false;
+  }
+  if (!grn_obj_have_source(ctx, obj)) {
+    return false;
+  }
+
+  return generator->length > 0;
 }
 
 static void
@@ -1408,11 +1470,11 @@ grn_obj_type_is_column(uint8_t type)
   }
 }
 
-grn_bool
+bool
 grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
 {
   if (!name) {
-    return GRN_FALSE;
+    return false;
   }
 
   if (name_len < 0) {
